// Title:	Material.cs
// Author: 	Scott Ellington <scott.ellington@gmail.com>
//
// Copyright (C) 2006 Scott Ellington and authors
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using CsGL.OpenGL;

namespace SalmonViewer
{
	public class Material
	{
		// Set Default values
		public float[] Ambient = new float [] { 0.5f, 0.5f, 0.5f };
		public float[] Diffuse = new float [] { 0.5f, 0.5f, 0.5f };
		public float[] Specular = new float [] { 0.5f, 0.5f, 0.5f };

		public int Shininess = 50;

        int textureid = -1;
		public int TextureId {
			get	{
				return textureid;
			}
		}

		//public void BindTexture ( int width, int height, byte [,,] data )
		public void BindTexture ( int width, int height, IntPtr data )
		{
			GL.glEnable( GL.GL_TEXTURE_2D );
			
			uint[] textures = new uint [1];
			GL.glGenTextures(1, textures);
			textureid = (int)textures[0];
			//Console.WriteLine ( "GL Texture number: {0}", textures [0] );

            GL.glBindTexture(GL.GL_TEXTURE_2D, (uint)textureid); 

			// repeat texture if neccessary
			//GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP); 
			//GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP);
			//GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_REPEAT);
			
			GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR); 
			//GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST); 
			//GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST); 
			GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR_MIPMAP_NEAREST); 

			// use GL_MODULATE to mix color with texture
			GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_MODULATE);
			//GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_REPLACE); 
			
			// Finally we define the 2d texture
			//GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGBA, width, height, 0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data);
			//GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGBA8, width, height, 0, GL.GL_BGRA_EXT, GL.GL_UNSIGNED_BYTE, data );
			GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, (int)GL.GL_RGBA8, width, height, 0, GL.GL_BGRA_EXT, GL.GL_UNSIGNED_BYTE, data );


            unsafe
            {
                // And create 2d mipmaps for the minifying function            
                CsGL.OpenGL.GLU.gluBuild2DMipmaps(GL.GL_TEXTURE_2D, 4, width, height, GL.GL_BGRA_EXT, GL.GL_UNSIGNED_BYTE, (void*)data);
                //       
                //GL.glTexImage2D(GL.GL_TEXTURE_2D,0, (int)GL.GL_RGB8, width, height, 0, GL.GL_BGR_EXT, GL.GL_UNSIGNED_BYTE, data);
            }

			GL.glDisable( GL.GL_TEXTURE_2D );
		}
      
	}
}
