﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
//using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections;

using System.IO;
using CsGL.OpenGL;
using CsGL.Util;

using SalmonViewer;



namespace gl
{




    public partial class Form1 : Form
    {

        #region declarações 

        public List<Triangulator.Geometry.Triangle> tris   = new List<Triangulator.Geometry.Triangle>();
        public List<Triangulator.Geometry.Point> points = new List<Triangulator.Geometry.Point>();

        public List<Model> models = new List<Model>();

        Image[] image = new Image[2];
        string[] imagefile = new string[2];

        int delta = 5; //tolerancia para selecionar um ponto

        bool creating = false;
        bool deleting = false;

        int selected = -1;
        int moving = -1;
        int width=1, height=1;

        private OGL glWindow = new OGL();
        
        public EditScene editForm = new EditScene();

        #endregion  declarações

        #region draw Images
        private void refreshImages()
        {
            

            Pen myPen = new Pen(Color.Blue, 1);

            PictureBox[] pb = new PictureBox[2];
            Graphics[] gr = new Graphics[2];
            Image[]  im = new Image[2];


            pb[0] = pictureBox1;
            pb[1] = pictureBox2;



            for (int i = 0; i < 2; i++)
            {

                im[i] = (Image)new Bitmap(pb[i].Width, pb[i].Height);
                gr[i] = Graphics.FromImage(im[i]);
                gr[i].InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.Low;
                if (image[i] != null) gr[i].DrawImage(image[i], 0, 0, pb[i].Width, pb[i].Height);
               
                // Perform Triangulation if there are more than 2 points in the collection
                if (points.Count > 2)

                    // Draw the created triangles
                    foreach (Triangulator.Geometry.Triangle t in tris)
                    {

                        gr[i].DrawLine(myPen, 
                            (float)points[t.p1].x[i] * pb[i].Width,
                            (float)points[t.p1].y[i] * pb[i].Height,
                            (float)points[t.p2].x[i] * pb[i].Width,
                            (float)points[t.p2].y[i] * pb[i].Height);
                        gr[i].DrawLine(myPen,
                            (float)points[t.p2].x[i] * pb[i].Width,
                            (float)points[t.p2].y[i] * pb[i].Height,
                            (float)points[t.p3].x[i] * pb[i].Width,
                            (float)points[t.p3].y[i] * pb[i].Height);
                        gr[i].DrawLine(myPen,
                            (float)points[t.p1].x[i] * pb[i].Width,
                            (float)points[t.p1].y[i] * pb[i].Height,
                            (float)points[t.p3].x[i] * pb[i].Width,
                            (float)points[t.p3].y[i] * pb[i].Height);

                      
                    }

            //Draw Points
            foreach (Triangulator.Geometry.Point p in points)
            {
                int v = 127 + (int)p.Z;
                if (v < 0) v = 0;
                if (v > 255) v = 255;

                Brush br = new SolidBrush(Color.FromArgb(v, v, v));
                Pen pen;
                if (p.selected) 
                pen= new Pen(Color.FromArgb(255, 255, 255));
                    else
                pen = new Pen(Color.FromArgb(255, 0, 0));

                gr[i].FillEllipse(br, (float)p.x[i] * pb[i].Width - 2, (float)p.y[i] * pb[i].Height - 2, 4, 4);
                gr[i].DrawEllipse(pen, (float)p.x[i] * pb[i].Width - 2, (float)p.y[i] * pb[i].Height - 2, 4, 4);

            }
            gr[i].Dispose();
            pb[i].Image = im[i];

            }
            ogl1.PlotGL();


        }
        #endregion   

        public Form1()
        {
            InitializeComponent();

            this.glWindow.Parent = this;
            this.glWindow.Top = 0; ; // fill the parent form

            this.glWindow.MouseMove += new System.Windows.Forms.MouseEventHandler(this.glWindow.glOnMouseMove);
            updateButtons();

        }

        private void Form1_Load_1(object sender, EventArgs e)
        {
            this.ogl1.MouseMove +=
            new System.Windows.Forms.MouseEventHandler(this.ogl1.glOnMouseMove);
            ogl1.models = models;
        }

        #region Points Manager
        //---------------------------------------------------------------------------
        private void addPoint(object sender, MouseEventArgs m)
        {
            PictureBox p = (PictureBox)sender;


            points.Add(new Triangulator.Geometry.Point((double)m.X / p.Width, (double)m.Y / p.Height));
            updatePoints();
            refreshImages();
        }
        private void delPoint(int id){

            points.Remove(points[id]);
            moving = selected = -1;
            updatePoints();
            refreshImages();
 
        }
        private void updatePoints()
        {

            //Calculate depths
            for (int i = 0; i < points.Count; i++)
                points[i].Z = points[i].X - points[i].X1;

            //Do triangulation
            tris.Clear();
            if (points.Count > 2)
                tris = Triangulator.Delauney.Triangulate(points);

            //Update GL Geometry
            if (points.Count > 2)
                ogl1.updateGeometry(points, tris);


        }

        private void bt_create_Click(object sender, EventArgs e)
        {
            creating = !creating;
            deleting = false;
            updateButtons();

        }
        private void bt_Delete_Click(object sender, EventArgs e)
        {
            deleting = !deleting;
            creating = false;
            updateButtons();
        }

        void updateButtons(){
          if (creating)
            {
                bt_create.ForeColor = Color.Red;
                bt_create.Text = "Stop Creating";
            }
            else
            {
                bt_create.ForeColor = Color.Black;
                bt_create.Text = "Create";
            }

          if (deleting)
          {
              bt_Delete.ForeColor = Color.Red;
              bt_Delete.Text = "Stop Deleting";
          }
          else
          {
              bt_Delete.ForeColor = Color.Black;
              bt_Delete.Text = "Delete";
          }

        }

        private void down(object sender, MouseEventArgs e)
        {
            if (creating)
                addPoint(sender, e);

            if (selected >= 0)
            {
                if (deleting)
                    delPoint(selected);
                else
                moving = selected;
            }
            
        }

        private void up(object sender, MouseEventArgs e)
        {
            moving = -1;
        }

        private void move(object sender, MouseEventArgs e)
        {
            PictureBox p = (PictureBox)sender;
            if (sender == pictureBox1) mmove(p, e, 0); else mmove(p, e, 1);
        }

        private void mmove(PictureBox p, MouseEventArgs e, int id)
        {
            if (moving >= 0)
            {
                points[moving].x[id] = (double)e.X / p.Width;
                points[moving].y[id] = (double)e.Y / p.Height;
                updatePoints();
            }
            else
            {
                double xu, xl, yu, yl;
                selected = -1;
                foreach ( Triangulator.Geometry.Point pt in points)
                {
                    xu = pt.x[id] * p.Width  + delta;
                    xl = pt.x[id] * p.Width  - delta;
                    yu = pt.y[id] * p.Height + delta;
                    yl = pt.y[id] * p.Height - delta;

                    if (xl < e.X && e.X < xu && yl < e.Y && e.Y < yu)
                    {
                        pt.selected = true;
                        selected = points.IndexOf(pt);
                    }
                    else
                        pt.selected = false;

                }

            }
            refreshImages();
        }

        #endregion MouseEvents

        #region image load

        void loadImage(string filename, int id)
        {
            if (filename == "" || filename==null) return;
            image[id] = new Bitmap(filename);
            Bitmap bit = new Bitmap(filename);
            imagefile[id] = filename;

            if (id == 0)
            {
                Color c;
                float[] colors = { 0, 0, 0 };
                width = image[id].Width;
                height = image[id].Height;
                ogl1.updateTexture(filename);

                int size = bit.Height * bit.Width;
                for (int i = 0; i < bit.Height; i++)
                {
                    for (int j = 0; j < bit.Height; j++)
                    {
                         c = bit.GetPixel(i, j);
                         colors[0] = colors[0] + c.R;
                         colors[1] = colors[1] + c.G;
                         colors[2] = colors[2] + c.B;
                    }
                }

                colors[0] = colors[0]/ size /256;
                colors[1] = colors[1]/ size / 256;
                colors[2] = colors[2]/ size/ 256;

                ogl1.average[0] = colors[0];
                ogl1.average[1] = colors[1];
                ogl1.average[2] = colors[2];

            }

            refreshImages();

        }

        private void loadl_Click(object sender, EventArgs e)
        {
            try
            {
                OpenFileDialog open = new OpenFileDialog();
                open.ShowDialog();
                loadImage(open.FileName, 0);
            }
            catch { }
        }

        private void loadr_Click(object sender, EventArgs e)
        {
                        try
            {
            OpenFileDialog open = new OpenFileDialog();
            open.ShowDialog();
            loadImage(open.FileName, 1);
            }
                        catch { }
        }
        #endregion

        #region Project Load

        private void button1_Click(object sender, EventArgs e)
        {

            OpenFileDialog open = new OpenFileDialog();
            open.ShowDialog();
            if (open.FileName != "")
            {
                points.Clear();
                tris.Clear();
                FileManager.load(open.FileName, points, imagefile);
            }
            loadImage(imagefile[0], 0);
            loadImage(imagefile[1], 1);
                
            updatePoints();
            refreshImages();

        }
        private void button2_Click(object sender, EventArgs e)
        {
            SaveFileDialog open = new SaveFileDialog();
            open.ShowDialog();
            if (open.FileName != "")
                FileManager.save(open.FileName, points, imagefile);
        }

        #endregion

        #region zooing
        private void bt_back_Click(object sender, EventArgs e)
        {
            //pictureBox1.Dock = DockStyle.None;
            //pictureBox2.Dock = DockStyle.None;
            ogl1.Dock = DockStyle.None;
            splitContainer1.Panel1Collapsed = false;
            bt_back.Visible = false;
            images.Visible = true;
            images.Panel1Collapsed = false;
            images.Panel2Collapsed = false;
            images.Dock = DockStyle.None;
            pictureBox2.Visible = true;
            pictureBox1.Visible = true;
            ogl1.Visible = true;

            refreshImages();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            
            //pictureBox1.Dock = DockStyle.Fill;
            //pictureBox2.Visible = false;
            images.Panel1Collapsed = false;
            images.Panel2Collapsed = true;
            images.Dock = DockStyle.Fill;
            ogl1.Visible = false;
            bt_back.Visible = true;
            splitContainer1.Panel1Collapsed = true;

            refreshImages();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            //pictureBox2.Dock = DockStyle.Fill;
            //pictureBox1.Visible = false;
            images.Panel1Collapsed = true;
            images.Panel2Collapsed = false;
            images.Dock = DockStyle.Fill;
            ogl1.Visible = false;
            bt_back.Visible = true;
            splitContainer1.Panel1Collapsed = true;

            refreshImages();
        }

        private void button5_Click(object sender, EventArgs e)
        {

            ogl1.Dock = DockStyle.Fill;
            ogl1.Visible = true;
            pictureBox1.Visible = false;
            pictureBox2.Visible = false;
            images.Visible = false;
            bt_back.Visible = true;
            splitContainer1.Panel1Collapsed = true;

            refreshImages();
        }


        private void bt_split_Click(object sender, EventArgs e)
        {
            pictureBox1.Dock = DockStyle.Fill;
            pictureBox2.Dock = DockStyle.Fill;
            bt_back.Visible = true;
            images.Dock = DockStyle.Fill;
            ogl1.Visible = false;
            refreshImages();
        }
        #endregion

        private void bt_editScene_Click(object sender, EventArgs e){

           
            editForm.ogl1 = ogl1;
            editForm.models = models;
            editForm.Show();
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            ogl1.distance =  (int)numericUpDown1.Value ;
            ogl1.updateGeometry(points,tris );
        }

        private void splitContainer1_Panel1_Paint(object sender, PaintEventArgs e)
        {
    

        }

        private void ch_wire_CheckedChanged(object sender, EventArgs e)
        {
                ogl1.wire = ch_wire.Checked;
                ogl1.shadows = check_shadows.Checked;
                ogl1.PlotGL();
        }

        private void button6_Click(object sender, EventArgs e)
        {
            ogl1.anglex = ogl1.angley = 0;
            ogl1.panx  = ogl1.pany = 0;
        }
  
    }
}
